/* Adam's GTK MOSIX monitor. This will run in a window if given no
   arguments, or as a GtkPlug if given an ID as an argument.

   mosmon is free software; you can redistribute and/or modify it
   under the terms of that license as published by the Free Software
   Foundation; either version 2 of the License, or (at your option)
   any later version.
   
   mosmon is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with GNU xhippo; see the file COPYING. If not, write to the
   Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307 USA, or see http://www.gnu.org/.
dd
*/

#include <linux/mosctl.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

/* The maximum load visible on each meter. */
#define SCALE 3

int numnodes, lastnumnodes = 0;
struct mosix_info info[MOSIX_MAX];
int ifd;

GtkWidget *window, *hbox;
GtkWidget *vbox[MOSIX_MAX], *bar[MOSIX_MAX], *label[MOSIX_MAX];

void read_info() {
	lseek(ifd, 0, 0);
	for (numnodes = 0;
		read(ifd, &info[numnodes], sizeof(struct mosix_info)) > 0;
		numnodes++)
		/**/ ;
}

void update_bars() {
	int i;

	if (lastnumnodes > numnodes) {
		for (i=numnodes; i<lastnumnodes; i++) {
			gtk_container_remove(GTK_CONTAINER(hbox), vbox[i]);
			gtk_container_remove(GTK_CONTAINER(vbox[i]), bar[i]);
			gtk_container_remove(GTK_CONTAINER(vbox[i]), label[i]);
			gtk_widget_destroy(vbox[i]);
			gtk_widget_destroy(bar[i]);
			gtk_widget_destroy(label[i]);
		}
	}

	if (numnodes > lastnumnodes) {
		for (i=lastnumnodes; i<numnodes; i++) {
			vbox[i] = gtk_vbox_new(FALSE, 0);

			bar[i] = gtk_progress_bar_new();
			gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(bar[i]), GTK_PROGRESS_CONTINUOUS);
			gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(bar[i]), GTK_PROGRESS_BOTTOM_TO_TOP);
			gtk_box_pack_start(GTK_BOX(vbox[i]), bar[i],
				TRUE, TRUE, 0);
			gtk_widget_show(bar[i]);
			
			label[i] = gtk_label_new("x");
			gtk_box_pack_end(GTK_BOX(vbox[i]), label[i],
				FALSE, FALSE, 0);
			gtk_widget_show(label[i]);

			gtk_widget_show(vbox[i]);

			gtk_box_pack_start(GTK_BOX(hbox), vbox[i],
				TRUE, TRUE, 0);
		}
	}

	lastnumnodes = numnodes;

	for (i=0; i<numnodes; i++) {
		float value = 0;
		char s[10];

		if (info[i].status & DS_MOSIX_UP) {
			value = info[i].load / (100.0 * SCALE);
			if (value > 1.0)
				value = 1.0;
			if (value < 0.0)
				value = 0.0;
			snprintf(s, 10, "%d", i + 1);
		} else {
			snprintf(s, 10, "X");
		}
		gtk_label_set_text(GTK_LABEL(label[i]), s);
		gtk_progress_set_percentage(GTK_PROGRESS(bar[i]), value);
	}
}

gint timeout(gpointer data) {
	read_info();
	update_bars();

	return 1;
}

gint handle_close(GtkWidget *widget, GdkEvent *event, gpointer data) {
	gtk_main_quit();
	return FALSE;
}

int main(int argc, char **argv) {
	gtk_init(&argc, &argv);

	ifd = open("/proc/mosix/info/infos", O_RDONLY);
	if (ifd < 0) {
		fprintf(stderr, "Can't open /proc/mosix/info/infos.\n");
		exit(20);
	}

	if (argc > 1) {
		window = gtk_plug_new(atoi(argv[1]));
	} else {
		window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(window), "mosmon");
	}

	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		GTK_SIGNAL_FUNC(handle_close), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(handle_close), NULL);
	gtk_widget_set_usize(window, 64, 64);

	hbox = gtk_hbox_new(TRUE, 0);
	gtk_container_add(GTK_CONTAINER(window), hbox);

	read_info();
	update_bars();

	gtk_widget_show(hbox);
	gtk_widget_show(window);

	gtk_timeout_add(500, timeout, NULL);

	gtk_main();

	return 0;
}
